using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using MikuMikuDance.XNA;
using MikuMikuDance.XNA.Model;
using MikuMikuDance.XNA.Accessory;
using MikuMikuDance.XNA.Motion;

namespace MikuMikuDanceXNADemo4
{
    //アクセサリ

    /// <summary>
    /// MikuMikuDance for XNAのデモ
    /// このデモでは
    /// -アクセサリの読み込み
    /// -アクセサリをモデルにセットする
    /// が書かれています
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        //MikuMikuDance for XNA
        MikuMikuDanceXNA mmd;
        //MMDモデルデータ
        MMDModel model;
        //MMDモーションデータ
        MMDBakedMotion motion;
        //MMDアクセサリデータ
        MMDAccessory stage, negi;

        public Game1()
        {
            //デバイスがMMDXの必要条件を満たしているかチェック
            graphics = new GraphicsDeviceManager(this);
            MikuMikuDanceXNA.CheckDevice(graphics);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent関数
        /// 各種ロード処理はここでロードを行っておくと良い
        /// </summary>
        protected override void LoadContent()
        {
            //MikuMikuDanceXNAを作成する
            mmd = new MikuMikuDanceXNA(this);
            //プロジェクトのContent内に追加してあるMiku.pmdモデルをロードする
            //ロードはMikuMikuDanceXNA.LoadModel関数で行う
            //(モデルデータはあにまさ氏よりお借りしました。このモデルファイルはMITライセンス適応外です)
            model = mmd.LoadModel("Miku", this, Matrix.Identity);
            //同じくContent内に追加してあるstage.x,negi.xアクセサリをロードする
            //(アクセサリデータはあにまさ氏よりお借りしました。このモデルファイルはMITライセンス適応外です)
            //(v1.1aではstage01のバックスクリーンに写す機能は実装していません。代わりにstage.bmpを入れています)
            stage = mmd.LoadAccessory("stage01", this, Matrix.Identity);//プロセッサの設定で14倍拡大してある
            negi = mmd.LoadAccessory("negi", this, Matrix.Identity);//プロセッサの設定で14倍拡大してある
            //アクセサリを持たせる用のnegi-vac.vacをロードする
            MMD_VAC vac = mmd.LoadVAC("negi-vac");
            //モデルにVACを指定することで持たせる(Matrixでも指定可)
            model.SetAccessory(negi, vac);

            //モーションロード
            motion = mmd.LoadBakedMotion("TrueMyHeartB");
            //モデルのプレイヤーを呼び出し
            //モーションを0番トラックにセットする
            model.Player.SetMotion(0, motion, true);
        }

        /// <summary>
        /// UnloadContent関数
        /// アンロードが必要な物はここでアンロードを行う
        /// </summary>
        protected override void UnloadContent()
        {
        }

        /// <summary>
        /// アップデート関数
        /// 毎フレーム処理する描画以外のコードをここに記述する
        /// </summary>
        /// <param name="gameTime">現在の時間のスナップショットを表すオブジェクト</param>
        protected override void Update(GameTime gameTime)
        {
            //XBoxコントローラーのBackボタンを押したら終了
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            //キーボードのEscキーを押したら終了
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();
            //エンターキーorAボタンを押したとき
            if (Keyboard.GetState().IsKeyDown(Keys.Enter) || GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed)
            {
                //現在再生中でなければ
                if (!model.Player.IsPlay(0))
                {
                    model.Player.Reset(0);//リセット
                    //0番トラックにセットされているモーションを再生する
                    model.Player.Start(0, false);
                }
            }

            //モデルの再生処理はDrawableGameComponentを継承しているため
            //XNAにより自動で処理が行われる

            //XNA内部のUpdate処理を行う
            base.Update(gameTime);
        }

        /// <summary>
        /// Draw関数
        /// 毎フレーム処理する描画コードをここに記述する
        /// </summary>
        /// <param name="gameTime">現在の時間のスナップショットを表すオブジェクト</param>
        protected override void Draw(GameTime gameTime)
        {
            //画面をCornflowerBlue色で消去する
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //モデルはDrawableGameComponentを継承しているため
            //XNAにより自動で描画される

            //XNA内部のDraw処理を行う
            base.Draw(gameTime);
        }
    }
}
